<?php

namespace App\Controller;

use App\Entity\Place;
use App\Entity\User;
use App\Form\PlaceType;
use App\Repository\PlaceRepository;
use App\Repository\TagRepository;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
use Doctrine\Common\Collections\Criteria;
use Doctrine\Common\Collections\ArrayCollection;



#[Route('/place')]
class PlaceController extends AbstractController
{
    #[Route('/', name: 'app_place_index', methods: ['GET'])]
    public function index(PlaceRepository $placeRepository): Response
    {

        $places = $placeRepository->findAll();
        $selected = 'name';
        $search = '';
        

        if (isset($_POST['card-type'])) {
            $cardType = $_POST['card-type'];
        }
        else {
            $cardType = 'card';
        }
        
        if (isset($_POST['search'])) {
            $search = $_POST['search'];
            $places = $placeRepository->search($search);
            $search = $_POST['search'];
        }

        if (isset($_POST['select-order'])) {
            $criteria = Criteria::create()->orderBy([$_POST['select-order'] => Criteria::ASC]);
            $placesCollection = new ArrayCollection($places);
            $places = $placesCollection->matching($criteria);
            $selected = $_POST['select-order'];
        }

        return $this->render('place/index.html.twig', [
            'places' => $places,
            'cardType' => $cardType,
            'selected' => $selected,
            'search' => $search
        ]);
    }

    #[Route('/{id}/favorite', name: 'app_place_favorite', methods: ['GET'])]
    public function favorite($id, UrlGeneratorInterface $urlGenerator, PlaceRepository $placeRepository, EntityManagerInterface $entityManager): Response
    {

        if (!$this->isGranted('IS_AUTHENTICATED_FULLY')) {
            throw new AccessDeniedException('You must be logged in to access this page.');
        }

        $place = $placeRepository->find($id);

        $place->setOrdered($place->getOrdered() + 1);
        $placeRepository->save($place, true);

        /** @var User $user */
        $user = $this->getUser();

        $place = $entityManager->getRepository(Place::class)->find($id);

        if (!$place || !$user) {
            return new Response('Could not find Place or User entity with the given ID(s)');
        }

        $user->addPlace($place);
        $place->addFkUser($user);

        $entityManager->flush();

        return new RedirectResponse($urlGenerator->generate('app_place_show', ['id' => $id]));
    }

    #[Route('/{id}/favorite/delete', name: 'app_place_favorite_delete', methods: ['GET'])]
    public function deleteFavorite($id, UrlGeneratorInterface $urlGenerator, PlaceRepository $placeRepository, EntityManagerInterface $entityManager): Response
    {
        if (!$this->isGranted('IS_AUTHENTICATED_FULLY')) {
            throw new AccessDeniedException('You must be logged in to access this page.');
        }

        /** @var User $user */
        $user = $this->getUser();

        $place = $entityManager->getRepository(Place::class)->find($id);

        if (!$place || !$user) {
            return new Response('Could not find Place or User entity with the given ID(s)');
        }

        $user->removePlace($place);
        $place->removeFkUser($user);

        $entityManager->flush();

        return new RedirectResponse($urlGenerator->generate('app_place_show', ['id' => $id]));
    }

    #[Route('/new', name: 'app_place_new', methods: ['GET', 'POST'])]
    public function new(Request $request, PlaceRepository $placeRepository, TagRepository $tagRepository): Response
    {

        $this->denyAccessUnlessGranted('ROLE_ADMIN');
        
        $place = new Place();
        $form = $this->createForm(PlaceType::class, $place);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $file = $request->files->get('place')['img'];
            $uploadsDirectory = $this->getParameter('uploads_directory');

            $fileName = md5(uniqid()) . '.' . $file->guessExtension();

            $file->move(
                $uploadsDirectory,
                $fileName
            );
            
            
            $place->setImg('/../uploads/' . $fileName);

            $place->setOrdered(0);

            $placeRepository->save($place, true);

            foreach($place->getTags() as $tag) {
                $tagRepository->save($tag, true);
                $tag->addFkPlace($place);
            }

            return $this->redirectToRoute('app_place_index', [], Response::HTTP_SEE_OTHER);
        }

        return $this->renderForm('place/new.html.twig', [
            'place' => $place,
            'form' => $form,
        ]);
    }

    #[Route('/{id}', name: 'app_place_show', methods: ['GET'])]
    public function show(Place $place, PlaceRepository $placeRepository): Response
    {
        $place->setViews($place->getViews() + 1);
        $placeRepository->save($place, true);

        // On regarde si la place est dans les favoris de l'utilisateur
        $isFavorite = false;
        if ($this->isGranted('IS_AUTHENTICATED_FULLY')) {
            $user = $this->getUser();
            $places = $user->getPlaces();
            foreach($places as $placeUser) {
                if ($placeUser->getId() == $place->getId()) {
                    $isFavorite = true;
                }
            }
        }

        return $this->render('place/show.html.twig', [
            'place' => $place,
            'isFavorite' => $isFavorite,
        ]);
    }

    #[Route('/{id}/edit', name: 'app_place_edit', methods: ['GET', 'POST'])]
    public function edit(Request $request, Place $place, PlaceRepository $placeRepository): Response
    {

        $this->denyAccessUnlessGranted('ROLE_ADMIN');

        $filePath = $place->getImg();

        $form = $this->createForm(PlaceType::class, $place, [
            'file_path' => $filePath,
        ]);

        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $placeRepository->save($place, true);

            return $this->redirectToRoute('app_place_index', [], Response::HTTP_SEE_OTHER);
        }

        return $this->renderForm('place/edit.html.twig', [
            'place' => $place,
            'form' => $form,
        ]);
    }

    #[Route('/{id}', name: 'app_place_delete', methods: ['POST'])]
    public function delete(Request $request, Place $place, PlaceRepository $placeRepository): Response
    {
        if ($this->isCsrfTokenValid('delete'.$place->getId(), $request->request->get('_token'))) {
            $placeRepository->remove($place, true);
        }

        return $this->redirectToRoute('app_place_index', [], Response::HTTP_SEE_OTHER);
    }
}
